﻿import numpy as np
import matplotlib.pyplot as plt
import agent
import society
import random as rnd

#term #期間
#population=x*y #人口
#edges #既存のノードに新たなノードから付け加わるリンク数
#initial_carries #初期感染者数
#beta #感染率(潜伏期間中)
#infection_rate #感染率(発症中)
#incubation_period #潜伏期間
#duration #罹患期間
#reinfection_rate #再感染率
#vaccination #予防接種率

def choose_initial_carriers(): #最初の感染者を決める
    initial_carriers_id=rnd.sample(range(population),k=initial_carriers)

    return initial_carriers_id

def choose_initial_vaccinators(): #最初のワクチン保持者を定める
    initial_vaccinators_id=rnd.sample(range(population),k=int(population*vaccination))

    return initial_vaccinators_id

def infection(agents): #感染
    for agent_id,agent in enumerate(agents): 
        #健常者
        if agent.state=='S':
            others_id=rnd.choice(agent.neighbors_id)
            #隣人の中からたまたま接触した相手を選ぶ
            others=agents[others_id]
            #接触した相手が感染者でかつ潜伏期である場合
            if others.state=='I':
                if rnd.random()<beta:
                    agent.state='I'
            #接触した相手が感染者でかつ発症中である場合
            elif others.state=='C':
                if rnd.random()<infection_rate:
                    agent.state='I'
        #発症中の感染者
        elif agent.state=='C':                  
            if agent.infection==duration:
                agent.state='R' #免疫保持者になる
                agent.infection=0
            else: #罹患期間を1日追加      
                agent.infection+=1 #pythonではincrement(++)やdecrement(--)は使えない
        #潜伏期間中の感染者
        elif agent.state=='I':
            #発症
            if agent.incubation==incubation_period:
                agent.state='C'
                agent.incubation=0
            else: #潜伏期間を1日追加
                agent.incubation+=1

        #免疫保持者
        elif agent.state=='R':
            others_id=rnd.choice(agent.neighbors_id)
            others=agents[others_id]
            #再感染
            if others.state=='I' or others.state=='C':
                if rnd.random()<reinfection_rate:
                    agent.state='I'
            else:
                pass
        else:
            continue

    return agents

def calc_carriers(agents):
    num_carriers=len([agent for agent in agents if agent.state=='C'])

    return num_carriers

def calc_incubation_infected(agents):
    num_incubation_infected=len([agent for agent in agents if agent.state=='I'])

    return num_incubation_infected

def square():
    global x
    global y
    j=0
    for i in range(1,population): 
        j=population/i
        if j>=i:
            x=int(i)
            y=int(j)
        else:
            continue

def run(term,input_beta,input_population,input_duration,input_incubation_period,input_initial_carriers,input_vaccination,input_reinfection_rate,input_edges,input_infection_rate,bool1):
    global beta 
    global population    
    global duration
    global incubation_period
    global initial_carriers
    global vaccination
    global reinfection_rate
    global edges
    global infection_rate
    beta=input_beta
    infection_rate=input_infection_rate
    population=input_population 
    duration=input_duration
    incubation_period=input_incubation_period
    edges=input_edges
    initial_carriers=input_initial_carriers
    vaccination=input_vaccination
    reinfection_rate=input_reinfection_rate 
    print("term:",term)
    print("population:",population)
    print("number of edges:",edges)
    print("number of initial carriers:",initial_carriers)  
    print("disease duration:",duration)    
    print("incubation period:",incubation_period)
    print("infection rate during incubation period:",beta)
    print("infection rate after onset")
    print("reinfection rate:",reinfection_rate) 
    print("vaccination rate:",vaccination)
    if bool1==False:
        agents=society.generate_agents(population,edges) #住人の生成
        rnd.seed() #乱数の初期設定
        #最初の感染者を選ぶ
        initial_carriers_id=choose_initial_carriers()
        for agent_id,agent in enumerate(agents):
            if agent_id in initial_carriers_id:
                agent.state='I' #感染者は潜伏期間中とする
        #最初のワクチン保持者を選ぶ
        initial_vaccinators_id=choose_initial_vaccinators()
        for agent_id,agent in enumerate(agents):
            if agent_id in initial_vaccinators_id:
                agent.state='R'
        #グラフの作成
        fig2=plt.figure()
        xlist=[]
        ylist=[]
        #感染シミュレーション
        for day in range(term):
            plt.clf() #グラフの更新        
            plt.axis([0,term,0,population])
            xlist.append(day)
            agents=infection(agents)
            num_carriers=calc_carriers(agents)
            #num_healthy=calc_healthy(agents)
            plt.title('epidemics')
            ylist.append(num_carriers)
            print("day:",day," number:",num_carriers)     
            plt.plot(xlist,ylist,'-',label='first')
            #plt.legend()
            plt.pause(0.01)
            if num_carriers==population:
                print("All have infected.")
                print("day:",day)
                plt.plot(term,num_carriers,'-',label='first')
            elif num_carriers==0:
                num_incubation_infected=calc_incubation_infected(agents)
                if num_incubation_infected==0:
                    print("Epidemic is over.")
                    print("day:",day)
                    plt.plot(term,num_carriers,'-',label='first')
                    break
                else:
                    continue
            else:
                continue
        
        plt.show()

    else:
        #2次元配列の表示
        print("2-D:",bool1)
        square()
        population=x*y #square()で定めたx,yから改めて近似の人口を定める
        agents=society.generate_agents(population,edges) #住人の生成
        rnd.seed() #乱数の初期設定
        #最初の感染者を選ぶ
        initial_carriers_id=choose_initial_carriers()
        for agent_id,agent in enumerate(agents):
            if agent_id in initial_carriers_id:
                agent.state='I' #感染者は潜伏期間中とする
        #最初のワクチン保持者を選ぶ
        initial_vaccinators_id=choose_initial_vaccinators()
        for agent_id,agent in enumerate(agents):
            if agent_id in initial_vaccinators_id:
                agent.state='R'
        #配列データの作成
        z=[[0 for i in range(x)] for j in range(y)]
        #print(z)
        fig=plt.figure() 
        ax=fig.add_subplot(111) 
        plt.title("epidemics")
        #感染シミュレーション
        for day in range(term):
            agents=infection(agents)
            num_carriers=calc_carriers(agents)
            print("day:",day," number:",num_carriers)
            z=[[0 for i in range(x)] for j in range(y)] #データの初期化
            for i in range(x): #データの更新
                for j in range(y):
                    if agents[i*y+j].state=='C':
                        z[j][i]=1
                    else:
                        pass
            ax.imshow(z,interpolation='nearest') 
            plt.pause(0.01)
            if num_carriers==population: #全員感染した場合
                print("All have infected.")
                print("day:",day)
            elif num_carriers==0:
                num_incubation_infected=calc_incubation_infected(agents)
                if num_incubation_infected==0:
                    print("Epidemic is over.")
                    print("day:",day)  
                    break
                else:
                    continue
            else:
                continue
            
        plt.show() 